/**
 * The CBC-MAC encryption module for NRF microcontrollers.
 *
 * @author  Ehsan Tofigh
 * @project lcx
 * @file    src/libcortex/ccm.c
 * @license
 */

#include <libcortex/ccm.h>
#include <libcortex/nvic.h>
#include <libcortex/util.h>

#include <string.h>

/** CCM engine state structure. */
static struct {
	uint8_t  key[16U];       /**< 16 byte AES key. */
	uint64_t cnt;            /**< Packet counter (ony 39 LSB bits are used) */
	struct {
		uint8_t dir : 1; /**< Direction. */
		uint8_t pad : 1; /**< Zero padding. */
		uint8_t     : 6;
	};
	uint8_t iv[8U];          /**< 8 byte initialisation vector. */
	uint8_t scratch[278U + 20];   /**< Scratch memory structure. */
} __attribute__((packed)) ccm;

bool lcx_ccm_init (const enum lcx_ccm_mode_datarate datarate,
		   const uint8_t                   *key,
		   const uint8_t                   *iv)
{
	lcx_assert(key == NULL);
	lcx_assert(iv == NULL);
	lcx_assert(datarate >= LCX_CCM_MODE_DATARATE_COUNT);

	/* Prepare the configuration structure: */
	memcpy(ccm.key, key, sizeof (ccm.key));
	memcpy(ccm.iv, iv, sizeof (ccm.iv));
	ccm.cnt = 0U;

	/*Initialise the CCM peripheral: */
	LCX_CCM->CNFPTR.CNFPTR         = (uint32_t) &ccm;
	LCX_CCM->SCRATCHPTR.SCRATCHPTR = (uint32_t) ccm.scratch;
	LCX_CCM->MODE.DATARATE         = lcx_ccm_mode_datarate_map[datarate];
	LCX_CCM->MODE.LENGTH           = lcx_ccm_mode_length_map[LCX_CCM_MODE_LENGTH_EXTENDED];
	// LCX_CCM->MAXPACKETSIZE.MAXPACKETSIZE = 0x1B;

	LCX_CCM->ENABLE.ENABLE = lcx_ccm_enable_enable_map[LCX_CCM_ENABLE_ENABLE_ENABLED];

	return true;
}

bool lcx_ccm_start (const enum lcx_ccm_mode_mode mode,
		    const uint8_t               *in,
		    const uint8_t               *out)
{
	lcx_assert(mode >= LCX_CCM_MODE_MODE_COUNT);
	lcx_assert(in == NULL);
	lcx_assert(out == NULL);

	LCX_CCM->MODE.MODE     = lcx_ccm_mode_mode_map[mode];
	LCX_CCM->INPTR.INPTR   = (uint32_t) in;
	LCX_CCM->OUTPTR.OUTPTR = (uint32_t) out;

	if (mode == LCX_CCM_MODE_MODE_ENCRYPTION) {
		lcx_set_shorts(LCX_CCM, .ENDKSGEN_CRYPT = 1U);
	} else {
		lcx_set_shorts(LCX_CCM, .ENDKSGEN_CRYPT = 0U);
	}

	return true;
}
